package com.fuzamei.service;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.fuzamei.dao.ProjectFileDao;
import com.fuzamei.entity.ProjectFile;
import com.fuzamei.utils.PageDTO;

@Service
public class ProjectFileService {

	@Autowired
	private ProjectFileDao projectFileDao;

	private static final int ROW_NUM = 10; // 分页每页显示数据的数量(固定10条数据)

	/**
	 * 
	* @Title: queryProjectFileInformation
	* @Description: TODO(这里用一句话描述这个方法的作用)
	* @param @param map
	* @param @return    设定文件
	* @return PageDTO    返回类型
	* @author ylx
	* @date 2017年12月15日 下午3:57:43
	* @throws
	 */
	public PageDTO queryProjectFileInformation(Map<String, Object> map) {
		int page = 1; // 默认页数是第一页
		String projectName = ""; // 默认名称为空值
		long startTime = 0; // 默认起始时间为0
		long endTime = Long.MAX_VALUE; // 默认终止时间为Long最大值
		if (!"".equals(map.get("page")) && map.get("page") != null) { // 等于空就直接取第一页
			try {
				page = Integer.parseInt((String) map.get("page"));
				if (page < 1) {
					page = 1;
				}
			} catch (Exception e) {
				page = 1; // 数据解析异常或者类型转换异常page还是1
			}
		}
		if (!"".equals(map.get("projectName")) && map.get("projectName") != null) { // 等于空就直接取空值
			try {
				projectName = (String) map.get("projectName");
			} catch (Exception e) {
				projectName="";
			}
		}
		if (!"".equals(map.get("startTime")) && map.get("startTime") != null) { // 等于空就直接取空值
			try {
				startTime = Long.parseLong((String) map.get("startTime"));
			} catch (Exception e) {
				startTime = 0; // 数据解析异常和类型转换异常startTime还是0
			}
		}
		if (!"".equals(map.get("endTime")) && map.get("endTime") != null) { // 等于空就直接取空值
			try {
				endTime = Long.parseLong((String) map.get("endTime"));
			} catch (Exception e) {
				endTime = Long.MAX_VALUE; // 数据解析异常和类型转换异常endTime还是9223372036854775807L
			}
		}

		PageDTO pageDto = new PageDTO(); // 创建分页对象
		Map<String, Object> mapToDao = new LinkedHashMap<String, Object>();
		mapToDao.put("startPage", (page - 1) * ROW_NUM);
		mapToDao.put("projectName", projectName);
		if (startTime <= endTime) {
			mapToDao.put("startTime", startTime);
			mapToDao.put("endTime", endTime);
		} else {
			mapToDao.put("startTime", startTime);
			mapToDao.put("endTime", Long.MAX_VALUE);
		}
		mapToDao.put("rowNum", ROW_NUM); // 默认每页显示数据是10条，可根据需求修改分页数量
		List<ProjectFile> informationList = projectFileDao.queryProjectFileInformation(mapToDao);
		int count = projectFileDao.findAllInformation();
		pageDto.setRows(informationList);
		pageDto.setTotal(count);
		return pageDto;
	}

	
	/**
	 * 
	* @Title: checkoutProjectFiles
	* @Description: TODO通过点击查看按钮，可以查看到这个项目名称下所有的项目文件信息以及hash值，并且提供文件的下载功能
	* @param @param map
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2017年12月15日 下午4:19:56
	* @throws
	 */
	public Map<String, Object> checkoutProjectFiles(Map<String, Object> map) {
		Integer projectId = null;	//默认项目编号为空
		String projectName="";		//默认项目名称为空值
		if(!"".equals(map.get("projectId")) && null!=map.get("projectId")){
			try {
				projectId = Integer.parseInt((String)map.get("projectId"));
			} catch (Exception e) {
				projectId = null;	//如果解析出现异常或者类型转换异常，依然保持projectId为null
			}
		}
		if (!"".equals(map.get("projectName")) && map.get("projectName") != null) { // 等于空就直接取空值
			try {
				projectName = (String) map.get("projectName");
			} catch (Exception e) {
				projectName="";
			}
		}
		
		Map<String, Object> mapToDao = new LinkedHashMap<String, Object>();
		
		mapToDao.put("projectId", projectId);
		mapToDao.put("projectName", projectName);
		
		List<ProjectFile> informationList = projectFileDao.checkoutProjectFiles(mapToDao);	//将项目id和项目名称封装成map到dao层去获取数据
		
		Map<String, Object> mapBackToController = new LinkedHashMap<String, Object>();
		
		mapBackToController.put("projectId", projectId);
		mapBackToController.put("projectName", projectName);
		mapBackToController.put("fileInformations", informationList);
		
		return mapBackToController;
	}

	/**
	 * 
	* @Title: insertInformationIntoTable
	* @Description: TODO(上传项目文档，不仅要将数据插入项目文档表和附件表，还要在操作记录中插入一条数据)
	* @param @param map    设定文件
	* @return void    返回类型
	* @author ylx
	* @date 2017年12月20日 下午8:16:14
	* @throws
	 */
	@Transactional(rollbackFor=Exception.class)
	public void insertInformationIntoTable(Map<String, Object> map) {
		String projectId="";			//项目id号
		String projectName="";			//项目文件名称
		String url="";					//url路径名称
		String attachmentName="";		//附件名称
		if (!"".equals(map.get("projectId")) && map.get("projectId") != null) { // 等于空就直接取空值
			try {
				projectId = (String) map.get("projectName");
			} catch (Exception e) {
				throw new RuntimeException("projectId类型转换异常");
			}
		}else{
			throw new RuntimeException("项目id号为空");
		}
		if (!"".equals(map.get("projectName")) && map.get("projectName") != null) { // 等于空就直接取空值
			try {
				projectName = (String) map.get("projectName");
			} catch (Exception e) {
				throw new RuntimeException("projectName类型转换异常");
			}
		}else{
			throw new RuntimeException("项目名称为空");
		}
		if (!"".equals(map.get("url")) && map.get("url") != null) { // 等于空就直接取空值
			try {
				url = (String) map.get("url");
			} catch (Exception e) {
				throw new RuntimeException("url类型转换异常");
			}
		}else{
			throw new RuntimeException("url路径名称为空");
		}
		if (!"".equals(map.get("attachmentName")) && map.get("attachmentName") != null) { // 等于空就直接取空值
			try {
				attachmentName = (String) map.get("attachmentName");
			} catch (Exception e) {
				throw new RuntimeException("attachmentName类型转换异常");
			}
		}else{
			throw new RuntimeException("附件名称为空");
		}
		
		//将数据封装到mapToDaoForProjectFile，并插入【项目文档表】
		Map<String, Object> mapToDaoForProjectFile = new LinkedHashMap<String,Object>();
		mapToDaoForProjectFile.put("projectId", projectId);
		mapToDaoForProjectFile.put("projectName", projectName);
		mapToDaoForProjectFile.put("attachmentId", (Integer)(int)(Math.random()*1000000));		//附件id不能用随机数，这里到时候要改的。。。。。。。。。TODO
		mapToDaoForProjectFile.put("updateTime", System.currentTimeMillis());			//上传文件的更新时间为当前时间
		mapToDaoForProjectFile.put("hash", "to be changed");							//上传文件的hash值到时候要通过对文件进行计算后才能进行数据库的录入(这里先写死)。。。。。。。。TODO
		projectFileDao.insertInformationIntoTable(mapToDaoForProjectFile);			//将以上的数据直接添加到数据库中【项目文档表】去
		
		//将数据封装到mapToDaoForAttachment，并插入【附件表】
		Map<String, Object> mapToDaoForAttachment = new LinkedHashMap<String,Object>();
		mapToDaoForAttachment.put("attachmentId", mapToDaoForProjectFile.get("attachmentId"));
		mapToDaoForAttachment.put("attachmentName", attachmentName);					//附件名称需要录入
		mapToDaoForAttachment.put("url", url);										//下载文件需要用到的url地址需要录入
		mapToDaoForAttachment.put("userId", 2000000001);								//用户id(这里先写死)。。。。。。。。。。TODO
		mapToDaoForAttachment.put("addTime", (Long)mapToDaoForProjectFile.get("updateTime"));		//这里添加的文件上传时间，和项目文档更新时间一样
		projectFileDao.insertInformationToAttachment(mapToDaoForAttachment);
		
		//此外，还要将业务申报的这个操作信息插入到操作记录表中【操作记录表】
		Map<String, Object> mapToOperationHistory = new LinkedHashMap<String,Object>();
		mapToOperationHistory.put("operatorTypeId", 1011);								//上传项目文档的类型为1011
		mapToOperationHistory.put("operatorAccount", 300000001);						//操作账号先定死
		mapToOperationHistory.put("operatorRole", "省分行托管分中心档案保管岗");			//创建这个操作的角色只能是省分行托管分中心档案保管岗
		mapToOperationHistory.put("operatorPerson", "张三");								//操作人暂时先写死
		mapToOperationHistory.put("operatorTime", (Long)mapToDaoForProjectFile.get("updateTime"));//操作时间为当前操作时间
		mapToOperationHistory.put("hash", "to be calculated from operation");			//如何计算操作的hash值？？？？
		projectFileDao.insertOperationHistoryInformationToDao(mapToOperationHistory);//将操作记录信息插入到操作记录表中
		
	}
	
	/*public static void main(String[] args) throws NoSuchAlgorithmException {
		MessageDigest complete=MessageDigest.getInstance("MD5");
		byte[] bs={1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
		complete.update(bs);
		byte[] digest = complete.digest();
		System.out.println(Arrays.toString(digest));
		System.out.println("=======================");
		System.out.println(Integer.toString(0x100));
		System.out.println(Integer.toString(36, 37));
		System.out.println(Integer.toString((16 & 0xff) + 0x100, 16).substring( 1 ));
		System.out.println(( 1 & 0xff ) + 0x100);
		System.out.println("=======================");
		
	}*/
	


	
}
