package com.fuzamei.service;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.fuzamei.dao.ValuationAccountingDao;
import com.fuzamei.entity.ValuationAccount;
import com.fuzamei.utils.PageDTO;

@Service
public class ValuationAccountingService {
	
	@Autowired
	private ValuationAccountingDao valuationAccountingDao;

	private static final int ROW_NUM = 10; // 分页每页显示数据的数量
	
	public PageDTO queryValuationAccountingInformation(Map<String, Object> map) {
		int page = 1; // 默认页是第一页
		String fundName="";	//基金名默认为空
		String battleResult="";	//比对结果默认为空
		long startTime = 0; // 开始时间默认0
		long endTime = Long.MAX_VALUE; // 结束时间默认Long最大值
		
		if (!"".equals(map.get("page")) && map.get("page") != null) { // 等于空就直接取第一页
			try {
				page = Integer.parseInt((String) map.get("page"));
				if (page < 1) {
					page = 1;
				}
			} catch (Exception e) {
				page = 1; // 数据解析异常或类型转换异常page还是1
			}
		}
		if (!"".equals(map.get("fundName")) && map.get("fundName") != null) { // 等于空就直接取空值
			try {
				fundName = (String) map.get("fundName");
			} catch (Exception e) {
				fundName="";//类型转换异常基金名称还是空字符串
			}
		}
		if (!"".equals(map.get("battleResult")) && map.get("battleResult") != null) { // 等于空就直接取空值
			try {
				battleResult = (String) map.get("battleResult");
			} catch (Exception e) {
				battleResult="";//类型转换异常比对结果还是空字符串
			}
		}
		if (!"".equals(map.get("startTime")) && map.get("startTime") != null) { // 等于空就直接取空值
			try {
				startTime = Long.parseLong((String) map.get("startTime"));
			} catch (Exception e) {
				startTime = 0; // 数据解析异常或者类型转换异常startTime还是0
			}
		}
		if (!"".equals(map.get("endTime")) && map.get("endTime") != null) { // 等于空就直接取空值
			try {
				endTime = Long.parseLong((String) map.get("endTime"));
			} catch (Exception e) {
				endTime = Long.MAX_VALUE; // 数据解析异常或者类型转换异常endTime还是9223372036854775807L
			}
		}
		Map<String, Object> mapToDao = new LinkedHashMap<String, Object>();				//这个map是去dao层调取数据的
		mapToDao.put("startPage", (page - 1) * ROW_NUM);
		mapToDao.put("fundName", fundName);
		mapToDao.put("battleResult", battleResult);
		
		//如果时间顺序错位，要重新进行排位
		if (startTime <= endTime) {
			mapToDao.put("startTime", startTime);
			mapToDao.put("endTime", endTime);
		} else {
			mapToDao.put("startTime", startTime);
			mapToDao.put("endTime", Long.MAX_VALUE);
		}
		mapToDao.put("rowNum", ROW_NUM); // 默认每页显示数据是10条，可根据需求修改分页数量
		
		PageDTO pageDTO=new PageDTO();
		
		List<ValuationAccount> informationList=valuationAccountingDao.queryValuationAccountingInformation(mapToDao);	//从service层中获取分页的对象信息
		int count=valuationAccountingDao.findAllInformations();	//获取总的信息数量
		pageDTO.setTotal(count);
		pageDTO.setRows(informationList);
		return pageDTO;
	}

	/**
	 * 
	* @Title: insertInformationIntoTable
	* @Description: TODO(以管理员身份将估值核算等数据插入数据库中)
	* @param @param map    设定文件
	* @return void    返回类型
	* @author ylx
	* @date 2017年12月18日 下午5:49:33
	* @throws
	 */
	@Transactional(rollbackFor=Exception.class)
	public void insertInformationIntoTableByAdmin(Map<String, Object> map) {
		String fundId="";			//基金id号
		String fundName="";			//基金名称
		String assets="";			//托管资产名称
		String url="";					//url路径名称
		String attachmentName="";		//附件名称
		if (!"".equals(map.get("fundId")) && map.get("fundId") != null) { // 等于空就直接取空值
			try {
				fundId = (String) map.get("fundId");
			} catch (Exception e) {
				throw new RuntimeException("fundId类型转换异常");
			}
		}else{
			throw new RuntimeException("项目id号为空");
		}
		if (!"".equals(map.get("fundName")) && map.get("fundName") != null) { // 等于空就直接取空值
			try {
				fundName = (String) map.get("fundName");
			} catch (Exception e) {
				throw new RuntimeException("fundName类型转换异常");
			}
		}else{
			throw new RuntimeException("项目名称为空");
		}
		if (!"".equals(map.get("assets")) && map.get("assets") != null) { // 等于空就直接取空值
			try {
				assets = (String) map.get("assets");
			} catch (Exception e) {
				throw new RuntimeException("assets类型转换异常");
			}
		}else{
			throw new RuntimeException("url路径名称为空");
		}
		if (!"".equals(map.get("url")) && map.get("url") != null) { // 等于空就直接取空值
			try {
				url = (String) map.get("url");
			} catch (Exception e) {
				throw new RuntimeException("url类型转换异常");
			}
		}else{
			throw new RuntimeException("url路径名称为空");
		}
		if (!"".equals(map.get("attachmentName")) && map.get("attachmentName") != null) { // 等于空就直接取空值
			try {
				attachmentName = (String) map.get("attachmentName");
			} catch (Exception e) {
				throw new RuntimeException("attachmentName类型转换异常");
			}
		}else{
			throw new RuntimeException("附件名称为空");
		}
		
		//将数据插入到估值核算表的数据封装到mapToDaoForValuationAccounting
		Map<String, Object> mapToDaoForValuationAccounting = new LinkedHashMap<String,Object>();
		mapToDaoForValuationAccounting.put("fundId", fundId);
		mapToDaoForValuationAccounting.put("fundName", fundName);
		mapToDaoForValuationAccounting.put("assets", assets);
		
		//管理员估值报表的id，而银行估值报表的id这个时候是没有添加的，也就是表中插入数据的时候，银行估值报表的id先空着
		mapToDaoForValuationAccounting.put("attachmentId", (Integer)(int)(Math.random()*1000000));		//附件id不能用随机数，这里到时候要改的(这个是管理员估值报表的id)。。。。。。。。。TODO
		mapToDaoForValuationAccounting.put("updateTime", System.currentTimeMillis());			//上传文件的更新时间为当前时间
		mapToDaoForValuationAccounting.put("battleResult", "待对账");							//上传文件的hash值到时候要通过对文件进行计算后才能进行数据库的录入(这里先写死)。。。。。。。。TODO
		valuationAccountingDao.insertInformationIntoTableByAdmin(mapToDaoForValuationAccounting);			//将以上的数据直接添加到数据库中【估值核算表】去

		//将附件信息放入到mapToDaoForAttachment并传递到数据库插入
		Map<String, Object> mapToDaoForAttachment = new LinkedHashMap<String,Object>();
		mapToDaoForAttachment.put("attachmentId", mapToDaoForValuationAccounting.get("attachmentId"));	//附件表的id直接从之前封装的map中获取即可
		mapToDaoForAttachment.put("attachmentName", attachmentName);									//附件名称需要录入
		mapToDaoForAttachment.put("url", url);															//下载文件需要用到的url地址需要录入
		//用户id到时候再确定，先写死
		mapToDaoForAttachment.put("userId", 2000000001);												//用户id(这里先写死)。。。。。。。。。。TODO
		mapToDaoForAttachment.put("addTime", (Long)mapToDaoForValuationAccounting.get("updateTime"));	//这里添加的文件上传时间，和估值核算添加的时间一致
		valuationAccountingDao.insertInformationToAttachment(mapToDaoForValuationAccounting);
	}

	/**
	 * 
	* @Title: updateInformationByBank
	* @Description: TODO(以银行的身份将银行的估值核算等数据更新到已经存在的表中,同时将上传估值核算的操作记录插入到操作记录表中去)
	* @param @param map    设定文件
	* @return void    返回类型
	* @author ylx
	* @date 2017年12月18日 下午8:21:51
	* @throws
	 */
	@Transactional(rollbackFor=Exception.class)
	public void updateInformationByBank(Map<String, Object> map) {
		String fundId="";			//基金id号
		String fundName="";			//基金名称
		String assets="";			//托管资产名称
		String url="";					//url路径名称
		String attachmentName="";		//附件名称
		if (!"".equals(map.get("fundId")) && map.get("fundId") != null) { // 等于空就直接取空值
			try {
				fundId = (String) map.get("fundId");
			} catch (Exception e) {
				throw new RuntimeException("fundId类型转换异常");
			}
		}else{
			throw new RuntimeException("项目id号为空");
		}
		if (!"".equals(map.get("fundName")) && map.get("fundName") != null) { // 等于空就直接取空值
			try {
				fundName = (String) map.get("fundName");
			} catch (Exception e) {
				throw new RuntimeException("fundName类型转换异常");
			}
		}else{
			throw new RuntimeException("项目名称为空");
		}
		if (!"".equals(map.get("assets")) && map.get("assets") != null) { // 等于空就直接取空值
			try {
				assets = (String) map.get("assets");
			} catch (Exception e) {
				throw new RuntimeException("assets类型转换异常");
			}
		}else{
			throw new RuntimeException("url路径名称为空");
		}
		if (!"".equals(map.get("url")) && map.get("url") != null) { // 等于空就直接取空值
			try {
				url = (String) map.get("url");
			} catch (Exception e) {
				throw new RuntimeException("url类型转换异常");
			}
		}else{
			throw new RuntimeException("url路径名称为空");
		}
		if (!"".equals(map.get("attachmentName")) && map.get("attachmentName") != null) { // 等于空就直接取空值
			try {
				attachmentName = (String) map.get("attachmentName");
			} catch (Exception e) {
				throw new RuntimeException("attachmentName类型转换异常");
			}
		}else{
			throw new RuntimeException("附件名称为空");
		}
		
		Map<String, Object> mapToDaoForValuationAccounting = new LinkedHashMap<String,Object>();
		mapToDaoForValuationAccounting.put("fundId", fundId);
		mapToDaoForValuationAccounting.put("fundName", fundName);
		mapToDaoForValuationAccounting.put("assets", assets);
		/*
		 * TODO
		 * 这里有一段代码是需要对管理岗和银行的上传估值报表进行比较，来确定battleResult的结果是---->>估值一致 or 估值不一致？？？？？？？？
		 */
		//这是一段伪代码，到时候要删除的
		if((int)(Math.random()*3)>1){
			mapToDaoForValuationAccounting.put("battleResult", "估值一致");	
		}else{
			mapToDaoForValuationAccounting.put("battleResult", "估值不一致");	
		}
		//管理员估值报表的id，而银行估值报表的id这个时候是没有添加的，也就是表中插入数据的时候，银行估值报表的id先空着
		mapToDaoForValuationAccounting.put("attachmentId", (Integer)(int)(Math.random()*1000000));		//附件id不能用随机数，这里到时候要改的(这个是银行估值报表的id)。。。。。。。。。TODO
		mapToDaoForValuationAccounting.put("updateTime", System.currentTimeMillis());			//上传文件的更新时间为当前时间
		valuationAccountingDao.updateInformationByBank(mapToDaoForValuationAccounting);			//将以上的数据更新到之前估值核算表
		
		
		//将附件信息封装到mapToDaoForAttachment并传入数据库中
		Map<String, Object> mapToDaoForAttachment = new LinkedHashMap<String,Object>();
		mapToDaoForAttachment.put("attachmentId", mapToDaoForValuationAccounting.get("attachmentId"));	//直接从前面的map中获取
		mapToDaoForAttachment.put("url", url);										//下载文件需要用到的url地址需要录入到附件表中
		mapToDaoForAttachment.put("attachmentName", attachmentName);					//附件名称需要录入
		//用户id到时候再确定，先写死
		mapToDaoForAttachment.put("userId", 2000000001);								//用户id(这里先写死)。。。。。。。。。。TODO
		mapToDaoForAttachment.put("addTime", (Long)mapToDaoForValuationAccounting.get("updateTime"));		//这里添加的文件上传时间，和项目文档更新时间一样
		
		
		//此外，还要将业务申报的这个操作信息插入到操作记录表中【】
		Map<String, Object> mapToOperationHistory = new LinkedHashMap<String,Object>();
		mapToOperationHistory.put("operatorTypeId", 1010);								//操作类型为【上传估值核算表】---1010
		mapToOperationHistory.put("operatorAccount", 300000001);						//操作账号先定死
		mapToOperationHistory.put("operatorRole", "省分行托管分中心估值核算岗");			//创建这个操作的角色只能是---省分行托管分中心估值核算岗
		mapToOperationHistory.put("operatorPerson", "张三");								//操作人先写死
		mapToOperationHistory.put("operatorTime", mapToDaoForValuationAccounting.get("updateTime"));			//操作时间为当前操作时间
		mapToOperationHistory.put("hash", "to be calculated from operation");			//如何计算操作的hash值？？？？
		valuationAccountingDao.insertOperationHistoryInformationToDao(mapToOperationHistory);//将操作记录信息插入到操作记录表中
		
		
		
	}

}
