package com.fuzamei.web;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.fuzamei.service.ProjectFileService;
import com.fuzamei.utils.JSONUtil;
import com.fuzamei.utils.PageDTO;

@Controller
@RequestMapping("/project_file")
public class ProjectFileAction {

	@Autowired
	private ProjectFileService projectFileService;

	private static final String SP = File.separator;
	/**
	 * 
	 * @Title: queryProjectFile 
	 * @Description:------------>>>>>>点击项目文档模块按钮的时候显示的分页信息&通过项目名称和时间区域的条件进行查询后显示的分页信息
	 * TODO(获取项目文档信息，用于展示项目文档基本信息展示) 
	 * 			以下为前端传递过来的参数
				 {
					"page":"1",
					"projectName":"",
					"startTime":"",
					"endTime":""
				}
	 * @param 
	 * @param data 
	 * @param @return
	 * 设定文件 @return Map<String,Object> 返回类型
	 * @author ylx 
	 * @date 2017年12月15日下午1:24:40
	 */
	@RequestMapping(value = "/queryProjectFileInformation")
	@ResponseBody
	public Map<String, Object> queryProjectFileInformation(@RequestBody String data) {

		System.out.println("查询项目文档记录信息....");

		Map<String, Object> map = JSONUtil.jsonToMap(data);

		PageDTO pageDto = projectFileService.queryProjectFileInformation(map); // 将前端传过来的json数据转化为map对象，并将map对象传入service层进行数据校验

		return JSONUtil.getJsonMap(200, true, "操作成功", pageDto);
	}
	
	/**
	 * 
	* @Title: checkoutProjectFiles
	* @Description: TODO--------------->>显示下载文件的提示框
	* @param @param data
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2017年12月15日 下午8:09:44
	* @throws
	 */
	@RequestMapping(value = "/checkoutProjectFiles")
	@ResponseBody
	public Map<String, Object> checkoutProjectFiles(@RequestBody String data){
		
		System.out.println("项目文档查看....");
		
		Map<String, Object> map = JSONUtil.jsonToMap(data);	// 将前端传过来的json数据转化为map对象，并将map对象传入service层进行数据校验
		
		Map<String, Object> mapFromService = projectFileService.checkoutProjectFiles(map); //由于这里不需要分页，只要service传过来一个map对象即可
		
		return JSONUtil.getJsonMap(200, true, "操作成功", mapFromService); //将从service返回回来的map对象通过map对象直接返回给前端
	}
	
	/**
	 * 
	* @Title: download
	* @Description: TODO(项目文档中的文件下载功能实现)
	* @param @param request
	* @param @param response
	* @param @param url    设定文件
	* @return void    返回类型
	* @author ylx
	* @date 2017年12月15日 下午6:03:53
	* @throws
	 */
	@RequestMapping(value="/downloadProjectFile")
	@ResponseBody
	public Map<String, Object> download(HttpServletRequest request,HttpServletResponse response,@RequestParam("url") String url) {
		try {
			String fileName = request.getServletContext().getRealPath("")+url;  		//SP是系统分隔符
			InputStream bis = new BufferedInputStream(new FileInputStream(new File(fileName)));  
			String filename = url.substring(url.lastIndexOf("/")+1);  //截取文件名
			filename = URLEncoder.encode(filename,"UTF-8");
			response.addHeader("Content-Disposition", "attachment;filename=" + filename);    
			response.setContentType("multipart/form-data");   
			BufferedOutputStream out = new BufferedOutputStream(response.getOutputStream());  
			int len = 0;  
			while((len = bis.read()) != -1){  
			    out.write(len);  
			    out.flush(); 
			}  
			out.close();
			bis.close();
			return JSONUtil.getJsonMap(200, true, "下载成功",null); //返回给前端一个map进行前段提示
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, "下载出现异常:"+e.getMessage(),null); //返回给前端一个map进行前段提示
		}
	}
	
	/**
	 * 
	* @Title: showUploadProjectFileInterface
	* @Description: TODO------------->>显示项目文档上传的界面
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2017年12月15日 下午8:11:09
	* @throws
	 */
	/*@RequestMapping(value = "/showUploadProjectFileInterface")
	@ResponseBody
	public Map<String, Object> showUploadProjectFileInterface(){
		
	}*/
	
	@RequestMapping(value="/uploadProjetcFile",method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> upload(@RequestParam("file") MultipartFile file,HttpServletRequest request,@RequestParam("projectId") String projectId,@RequestParam("projectName") String projectName ) {
		
//		Map<String, Object> map = JSONUtil.jsonToMap(data);	// 将前端传过来的json数据转化为map对象，并将map对象传入service层进行数据校验
		
		try {
			String path = request.getServletContext().getRealPath("")+SP+"content";			//SP是系统分隔符	
			File dir=new File(path);
			if(!dir.exists()){
				dir.mkdirs();
			}
			
			String filename = file.getOriginalFilename();
			String pathFile=path+SP+filename;
			File newFile=new File(pathFile);
			file.transferTo(newFile);
			
			Map<String, Object> map =new LinkedHashMap<String, Object>();
			
			map.put("projectId", projectId);
			map.put("projectName", projectName);
			map.put("url", "/content/"+filename);		//将文件下载路径url通过map传到service层进行校验
			map.put("attachmentName", filename);			//将文件以附件名的形式通过map传递到service层中进行校验
			
			projectFileService.insertInformationIntoTable(map);		//将获取到的数据插入到表中
			
			Map<String, Object> mapResult = JSONUtil.getJsonMap(200, true, "上传成功",null); //返回给前端一个map进行前段提示
			return mapResult;
		} catch (Exception e) {
			Map<String, Object> mapResult = JSONUtil.getJsonMap(500, true, "上传出现异常"+e.getMessage(),null); //返回给前端一个map进行前段提示
			return mapResult;
		}
		
	}
	
}
