package com.fuzamei.web;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.fuzamei.service.ValuationAccountingService;
import com.fuzamei.utils.JSONUtil;
import com.fuzamei.utils.MapUtil;
import com.fuzamei.utils.PageDTO;

/**
 * 估值核算模块实现
 * @author ylx
 *
 */
@Controller
@RequestMapping("valuation_accounting")
public class ValuationAccountingAction {

	@Autowired
	private ValuationAccountingService valuationAccountingService;
	
	private static final String SP = File.separator;	//系统分隔符，随操作系统而决定
	
	/**
	 * 
	* @Title: queryOperationHistoryInformation
	* @Description: TODO(估值核算的查询结果，包括所有的查询条件也包括进去了)
	* @param @param data
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2017年12月18日 下午3:17:30
	* @throws
	 */
	@RequestMapping(value="/queryValuationAccountingInformation",method=RequestMethod.POST)
	@ResponseBody
	public Map<String,Object> queryOperationHistoryInformation(@RequestBody String data){
		
		System.out.println("查询估值核算数据");
		
		Map<String, Object> map = JSONUtil.jsonToMap(data);
		
		PageDTO pageDto = valuationAccountingService.queryValuationAccountingInformation(map);
		
		return JSONUtil.getJsonMap(200, true, "查询成功", pageDto);
	}
	
	/**
	 * 
	* @Title: download
	* @Description: TODO(估值核算界面，下载功能实现)
	* @param @param request
	* @param @param response
	* @param @param url
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2017年12月18日 下午5:26:46
	* @throws
	 */
	@RequestMapping(value="/downloadValuationPOI")
	@ResponseBody
	public Map<String, Object> download(HttpServletRequest request,HttpServletResponse response,@RequestParam("url") String url) {
		try {
			String fileName = request.getServletContext().getRealPath("")+url;  		//SP是系统分隔符
			InputStream bis = new BufferedInputStream(new FileInputStream(new File(fileName)));  
			String filename = url.substring(url.lastIndexOf("/")+1);  //截取文件名
			filename = URLEncoder.encode(filename,"UTF-8");
			response.addHeader("Content-Disposition", "attachment;filename=" + filename);    
			response.setContentType("multipart/form-data");   
			BufferedOutputStream out = new BufferedOutputStream(response.getOutputStream());
			int len = 0;  
			while((len = bis.read()) != -1){  
			    out.write(len);  
			    out.flush(); 
			}  
			out.close();
			bis.close();
			return JSONUtil.getJsonMap(200, true, "下载成功",null); //返回给前端一个map进行前段提示
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, "下载出现异常:"+e.getMessage(),null); //返回给前端一个map进行前段提示
		}
	}
	
	/**
	 * 
	* @Title: upload
	* @Description: TODO(实现管理员估值核算的报表文件上传功能---------->>暂时按照一次上传一个文件进行实现)
	* 
	* 如果是管理岗的人员进行上传，则是创建一条新的数据进数据库，同时在附件表中插入一条新的数据
	* 
	* @param @param file
	* @param @param request
	* @param @param fundId 			基金id号，前端随机生成？？？？？还是说后台生成？？？？TODO
	* @param @param fundName		基金名称，客户端手动输入生成并传输过来
	* @param @param assets			托管资产，客户端手动输入生成并传输过来
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2017年12月18日 下午5:29:38
	* @throws
	 */
	@RequestMapping(value="/uploadValuationPOIByAdmin",method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> uploadValuationPOIByAdmin(@RequestParam("file") MultipartFile file,HttpServletRequest request,@RequestParam("fundId") String fundId,@RequestParam("fundName") String fundName, @RequestParam("assets") String assets ) {
		try {
			String path = request.getServletContext().getRealPath("")+SP+"content";			//SP是系统分隔符	
			File dir=new File(path);
			if(!dir.exists()){
				dir.mkdirs();
			}
			
			String filename = file.getOriginalFilename();
			String pathFile=path+SP+filename;
			File newFile=new File(pathFile);
			file.transferTo(newFile);
			
			Map<String, Object> map =new LinkedHashMap<String, Object>();
			map.put("fundId", fundId);
			map.put("fundName", fundName);
			map.put("assets", assets);
			map.put("url", "/content/"+filename);		//将文件下载路径url通过map传到service层进行校验
			map.put("attachmentName", filename);			//将文件以附件名的形式通过map传递到service层中进行校验
			
			//这里是以管理岗的身份进行插入的，注意银行的估值报表暂时先不插入数据
			valuationAccountingService.insertInformationIntoTableByAdmin(map);		//将获取到的数据插入到表中
			
			return JSONUtil.getJsonMap(200, true, "上传成功",null); //返回给前端一个map进行前段提示
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, "上传出现异常"+e.getMessage(),null); //返回给前端一个map进行前段提示
		}
	}
	
	/**
	 * 
	* @Title: upload
	* @Description: TODO(实现银行估值核算的报表文件上传功能---------->>暂时按照一次上传一个文件进行实现)
	* @param @param file
	* @param @param request
	* @param @param fundId
	* @param @param fundName
	* @param @param assets
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2017年12月18日 下午8:17:47
	* @throws
	 */
	@RequestMapping(value="/uploadValuationPOIByBank",method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> uploadValuationPOIByBank(@RequestParam("file") MultipartFile file,HttpServletRequest request,@RequestParam("fundId") String fundId,@RequestParam("fundName") String fundName, @RequestParam("assets") String assets ) {
		try {
			String path = request.getServletContext().getRealPath("")+SP+"content";			//SP是系统分隔符	
			File dir=new File(path);
			if(!dir.exists()){
				dir.mkdirs();
			}
			String filename = file.getOriginalFilename();
			String pathFile=path+SP+filename;
			File newFile=new File(pathFile);
			file.transferTo(newFile);					//这里已经完成银行估值文件的上传，后续需要对该文件和管理岗上传文件中的报价信息进行比对，在Service层中进行校验
			
			Map<String, Object> map =new LinkedHashMap<String, Object>();
			map.put("fundId", fundId);
			map.put("fundName", fundName);
			map.put("assets", assets);
			map.put("url", "/content/"+filename);		//将文件下载路径url通过map传到service层进行校验
			map.put("attachmentName", filename);			//将文件以附件名的形式通过map传递到service层中进行校验
			
			//这里是以银行的身份将数据进行更新
			valuationAccountingService.updateInformationByBank(map);		//将获取到的数据更新到已有的估值核算表中，这个表是由管理岗创建的
			
			return JSONUtil.getJsonMap(200, true, "上传成功",null); //返回给前端一个map进行前段提示
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, "上传出现异常"+e.getMessage(),null); //返回给前端一个map进行前段提示
		}
	}
	
	
}
